#!/usr/bin/perl -w
# Chequeo pasivo generico
# Lee los servicios a chequear de /etc/nagios/check_nsca.conf
# Sustituye las variables $VAR$ por los valores pasados por linea de comandos
#
# 1.2: 2024-02-08 daniel
#      Corrijo parseo de opciones
#      Creo nuevo check_nrdp.pl basado en el check_nsca.pl.
#      Cambio de nombre archivos de servicios para que no dependa de si se usa NSCA o NRDP
#
# 1.1: 2014-05-14 daniel
#      Uso &print_version para version
#      Agrego opcion -P para el puerto del NSCA
#      Agrego chequeo 'Uptime' con el uptime de la maquina
#
# 1.0: 2014-05-05 daniel
#      Version inicial

use strict;
use Net::SNMP qw(:snmp);
use Data::Dumper;
use Getopt::Long;
use lib "/usr/lib64/nagios/plugins";
use utils qw($TIMEOUT %ERRORS);
use ipsupervisor qw(&send_nsca &round &check_drbd &read_config &print_version
                    $nagios_hostname $nagios_hostname_ha $nsca_host $nsca_port $debug $snmp_timeout $nrdp_url $nrdp_token);

my ($opt_h, $opt_V, $opt_v, $opt_N, $opt_H, $opt_A, $opt_C, $opt_S, $opt_d);
my $PROGNAME = $0;

sub print_usage();

&read_config();
$opt_H = $nagios_hostname if $nagios_hostname;
$opt_A = $nagios_hostname_ha if $nagios_hostname_ha;
$opt_N = $nrdp_url if $nrdp_url;

Getopt::Long::Configure('bundling');
GetOptions
    ("h"   => \$opt_h, "help"           => \$opt_h,
     "V"   => \$opt_V, "version"        => \$opt_V,

     "N=s" => \$opt_N, "nrdp=s"         => \$opt_N,
     "H=s" => \$opt_H, "hostname=s"     => \$opt_H,
     "A=s" => \$opt_A, "hahostname=s"   => \$opt_A,
     "C=s" => \$opt_C, "community=s"    => \$opt_C,
     "v=s" => \$opt_v, "snmp_version=s" => \$opt_v,
     "S=s" => \$opt_S, "snmp=s"         => \$opt_S,
     "d"   => \$opt_d, "debug"          => \$opt_d);

if ($opt_V) {
    print_version($PROGNAME,'1.2');
    exit $ERRORS{'OK'};
}

if ($opt_h) {
	print_usage();
	exit $ERRORS{'OK'};
}

# Hostname Centreon (requerido)
(print("Nagios hostname required!\n") && print_usage() && exit $ERRORS{'UNKNOWN'}) unless ($opt_H);

# NRDP Url (requerido)
(print("NRDP Url required!\n") && print_usage() && exit $ERRORS{'UNKNOWN'}) unless ($opt_N);
#print "Url: $opt_N\n";

## Opcionales
# Host SNMP
($opt_S) || ($opt_S = "localhost");

# Community
($opt_C) || ($opt_C = "ipcontact");

# Version SNMP
($opt_v) || ($opt_v = "1");
my $snmp = $1 if ($opt_v =~ /(\d)/);

# Debug
$debug = $opt_d;

my $line;
my @chequeos;
my ($service,$check_cmd);
open(CONFIG, "/etc/nagios/services.conf") || die "Error al abrir configuracion!\n";
while ($line = <CONFIG>) {
    next if ($line =~ /^\s*\#\w*/);
    #($service, $check_cmd) = ($1,$2) if ($line =~ /^(\w+)\|(.+)$/);
    ($service, $check_cmd) = ($1,$2) if ($line =~ /^([^|]+)\|(.+)$/);
    # Sustituyo variables
    $check_cmd =~ s/\$HOSTADDRESS\$/$opt_S/g;
    $check_cmd =~ s/\$HOSTSNMPCOMMUNITY\$/$opt_C/g;
    $check_cmd =~ s/\$HOSTSNMPVERSION\$/$opt_v/g;
    my %serv = (
	cmd => $check_cmd,
	service => $service
    );
    push @chequeos,\%serv;
    #print "service: $service, cmd: $check_cmd\n";
}
close CONFIG;
#print Dumper @chequeos;

my $drbd = &check_drbd;
my @chequeos_ha;
if ($drbd == 1) {
    my $exists = -r "/etc/nagios/services_ha.conf";
    if ($exists) {
	open(CONFIG, "/etc/nagios/services_ha.conf") || die "Error al abrir configuracion HA!\n";
	while ($line = <CONFIG>) {
	    next if ($line =~ /^\s*\#\w*/);
	    #($service, $check_cmd) = ($1,$2) if ($line =~ /^(\w+)\|(.+)$/);
    	    ($service, $check_cmd) = ($1,$2) if ($line =~ /^([^|]+)\|(.+)$/);
	    # Sustituyo variables
	    $check_cmd =~ s/\$HOSTADDRESS\$/$opt_S/g;
	    $check_cmd =~ s/\$HOSTSNMPCOMMUNITY\$/$opt_C/g;
	    $check_cmd =~ s/\$HOSTSNMPVERSION\$/$opt_v/g;
	    my %serv = (
		cmd => $check_cmd,
		service => $service
		);
	    push @chequeos_ha,\%serv;
	    #print "service: $service, cmd: $check_cmd\n";
	}
	close CONFIG;
    }
}
#print Dumper @chequeos_ha;

my ($cmd, $res, $status);
my $nrdp_str;
foreach my $c (@chequeos) {
    # $USER1$/check_centreon_snmp_loadaverage -H $HOSTADDRESS$ -v $_HOSTSNMPVERSION$ -C $_HOSTSNMPCOMMUNITY$ -w $ARG1$ -c $ARG2$che
    #print "* Chequeo: " . $c->{service} . "\n";
    $cmd = "/usr/lib64/nagios/plugins/" . $c->{cmd};
    $res = `$cmd`;
    $status = $? >> 8;
    #print "$c->{service}";
    chomp($res);
    $nrdp_str = "$opt_H|" . $c->{service} . "|" . $status . "|$res\n";
    print "send_nrdp: $nrdp_str" if $debug;
    system("/usr/bin/send_nrdp.sh -u $opt_N -t $nrdp_token -H $opt_H -s \"$c->{service}\" -S $status -o \"$res\"");
    print "\n" if $debug;
}

if ($drbd == 1) {
    # Indica cual es hostname con HA
    my $nodo = substr $nagios_hostname, -1;
    system("/usr/bin/send_nrdp.sh -u $opt_N -t $nrdp_token -H $opt_A -s \"Nodo_HA\" -S 0 -o \"Activo: $nagios_hostname | activo=$nodo\"");

    foreach my $c (@chequeos_ha) {
	#print "* Chequeo: " . $c->{service} . "\n";
        $cmd = "/usr/lib64/nagios/plugins/" . $c->{cmd};
	$res = `$cmd`;
	$status = $? >> 8;
	chomp($res);
	#print "Res: $res, status $status\n";
	$nrdp_str = "$opt_H|" . $c->{service} . "|" . $status . "|$res\n";
	print "send_nrdp: $nrdp_str" if $debug;
        system("/usr/bin/send_nrdp.sh -u $opt_N -t $nrdp_token -H $opt_A -s \"$c->{service}\" -S $status -o \"$res\"");
	print "\n" if $debug;
    }
}



sub print_usage () {
    print "\nUsage:\n";
    print "$PROGNAME\n";
    print "   -U (--nrdp)          NRDP Url to send passive check (required, or read from ipsupervisor.conf)\n";
    print "   -H (--hostname)      Nagios hostname (required, or read from ipsupervisor.conf )\n";
    print "   -A (--hahostname)    Nagios HA hostname (optional)\n";
    print "   -P (--port)          NSCA port to send passive check (default 5667)\n";
    print "   -S (--snmp)          SNMP hostname to query - (defaults to 'localhost')\n";
    print "   -C (--community)     SNMP read community (defaults to 'ipcontact',\n";
    print "                        used with SNMP v1 and v2c\n";
    print "   -v (--snmp_version)  1 for SNMP v1 (default)\n";
    print "                        2 for SNMP v2c\n";
    print "   -d (--debug)         Enable NSCA debug (default disabled)\n";
    print "   -V (--version)       Plugin version\n";
    print "   -h (--help)          usage help\n";

}

