#!/usr/bin/perl -w
# Chequeo pasivo para IPContact
#
#
# 1.2: 2024-02-08 daniel
#      Corrijo parseo de opciones
#      Creo nuevo check_nrdp_ipcontact.pl basado en el check_nsca_ipcontact.pl
#
# 1.1: 2014-05-14 daniel
#      Uso &print_version para version
#      Agrego opcion -P para el puerto del NSCA
#      En el chequeo 'IPCONTACT_Uptime' mando uptime_mins. Estados warning/critical segun el valor de uptime del jastserv
#      Por defecto no hay estados warning/critical para el chequeo 'LINEAS_*'
#
# 1.0: 2014-05-05 daniel
#      Version inicial

use strict;
use Net::SNMP qw(:snmp);
use Data::Dumper;
use Getopt::Long;
use lib "/usr/lib64/nagios/plugins";
use utils qw($TIMEOUT %ERRORS);
use ipsupervisor qw(&send_nsca &round &check_drbd &read_config &print_version
                    $nagios_hostname $nagios_hostname_ha $nsca_host $nsca_port $debug $snmp_timeout $nrdp_url $nrdp_token);

my ($opt_h, $opt_V, $opt_v, $opt_N, $opt_H, $opt_C, $opt_S, $opt_p, $opt_d, $opt_w, $opt_c);
my $PROGNAME = $0;

# Salgo si hay config DRBD y no esta activo
exit 0 unless &check_drbd();

sub print_usage();

&read_config();
if ($nagios_hostname_ha) {
    $opt_H = $nagios_hostname_ha;
} else {
    $opt_H = $nagios_hostname if $nagios_hostname;
}
$opt_N = $nrdp_url if $nrdp_url;

Getopt::Long::Configure('bundling');
GetOptions
    ("h"   => \$opt_h, "help"           => \$opt_h,
     "V"   => \$opt_V, "version"        => \$opt_V,

     "N=s" => \$opt_N, "nrdp=s"         => \$opt_N,
     "H=s" => \$opt_H, "hostname=s"     => \$opt_H,
     "C=s" => \$opt_C, "community=s"    => \$opt_C,
     "v=s" => \$opt_v, "snmp_version=s" => \$opt_v,
     "S=s" => \$opt_S, "snmp=s"         => \$opt_S,
     "p=s" => \$opt_p, "prefix=s"       => \$opt_p,
     "d"   => \$opt_d, "debug"          => \$opt_d,
     "w=s" => \$opt_w, "warning=s"      => \$opt_w,
     "c=s" => \$opt_c, "critical=s"     => \$opt_c);

if ($opt_V) {
    print_version($PROGNAME,'1.2');
    exit $ERRORS{'OK'};
}

if ($opt_h) {
	print_usage();
	exit $ERRORS{'OK'};
}

# Hostname Centreon (requerido)
(print("Nagios hostname required!\n") && print_usage() && exit $ERRORS{'UNKNOWN'}) unless ($opt_H);

# NRDP Url (requerido)
(print("NRDP Url required!\n") && print_usage() && exit $ERRORS{'UNKNOWN'}) unless ($opt_N);

## Opcionales
# Warning
my $warn;
if ($opt_w) {
    $warn = $1 if ($opt_w =~ /^([0-9]{1,2}\%|100\%)$/);
    ($warn) || (print("Invalid warning threshold: $opt_w\n") && exit $ERRORS{'UNKNOWN'});
    $warn =~ s/\%//g;
}

# Critical
my $crit;
if ($opt_c) {
    $crit = $1 if ($opt_c =~ /^([0-9]{1,2}\%|100\%)$/);
    ($crit) || (print("Invalid critical threshold: $opt_c\n") && exit $ERRORS{'UNKNOWN'});
    $crit =~ s/\%//g;
    unless ($opt_w) {
	print("You must specify both warning and critical, or neither of them for no warning/critical states\n") && exit $ERRORS{'UNKNOWN'};
    }
} else {
    if ($opt_w) {
	print("You must specify both warning and critical, or neither of them for no warning/critical states\n") && exit $ERRORS{'UNKNOWN'};
    }
}

if ($opt_w && $opt_c && ($warn > $crit)) {
    (print("Error: critical ($opt_c) should be greater than warning ($opt_w) \n") && exit $ERRORS{'UNKNOWN'});
}

# Host SNMP
($opt_S) || ($opt_S = "localhost");

# Community
($opt_C) || ($opt_C = "ipcontact");

# Version SNMP
($opt_v) || ($opt_v = "1");
my $snmp = $1 if ($opt_v =~ /(\d)/);

# Prefijo
($opt_p) || ($opt_p = "LINEAS_");

# Debug
$debug = $opt_d;

my ( $session, $error ) = Net::SNMP->session(-hostname  => $opt_S, -community => $opt_C, -version  => $opt_v, -timeout => $snmp_timeout);
if ( !defined($session) ) {
    print("CRITICAL: SNMP Session : $error");
    # Ver que mandar en este caso
    exit $ERRORS{'UNKNOWN'};
}

# OIDs
# Tablas, subindice extra
my $oid_eliName = "1.3.6.1.4.1.40851.4.1.1.1.1";
my $oid_eliSize = "1.3.6.1.4.1.40851.4.1.1.1.2";
my $oid_eliOutgoingTot = "1.3.6.1.4.1.40851.4.1.1.1.3";
my $oid_eliIncomingTot = "1.3.6.1.4.1.40851.4.1.1.1.7";
# Llamadas locales
my $oid_lciCalls = "1.3.6.1.4.1.40851.4.1.2.1.1.1";

# Jastserv
my $oid_serverUptime = "1.3.6.1.4.1.40851.3.2.0";
my $oid_licence_status = "1.3.6.1.4.1.40851.3.3.1.0";

my @grupos;
my %datos;
my $c = 0;
my ($tot_in, $tot_out, $tot_max);


my ($res_grupos, $res_size, $res_out, $res_in);
# Recabo toda la informacion sobre las lineas
$res_grupos = $session->get_table($oid_eliName);
if (defined $res_grupos) {
    $res_size = $session->get_table($oid_eliSize);
    $res_out = $session->get_table($oid_eliOutgoingTot);
    $res_in = $session->get_table($oid_eliIncomingTot);
} else {
    print "Warning!: No response from SNMP server, no line info\n";
}
#print Dumper $res_grupos;

# Llamadas locales
my $res_locales = $session->get_request($oid_lciCalls);
#print Dumper($res_locales);
# Uptime Jastserv
my $res_uptime = $session->get_request($oid_serverUptime);
# Estado Licencia Jastserv
my $res_licence = $session->get_request($oid_licence_status);


## Armo una estructura con los datos recabados
my %res_grupos_h;
# Hago esto porque si no $res_grupos pasa a ser vacio luego del foreach
%res_grupos_h = %$res_grupos if defined $res_grupos;
foreach my $key (sort keys %res_grupos_h) {
    my %datos;
    $datos{'name'} = $res_grupos->{$key} ;
    push @grupos,\%datos;
    #print "key: $key => ";
    #print $res_grupos->{$key} ."\n";
}

# Size
$c = 0;
foreach my $key (sort keys %$res_size) {
    $grupos[$c]->{'size'} = $res_size->{$key};
    $c++;
}
# Outgoing
$c = 0;
foreach my $key (sort keys %$res_out) {
    $grupos[$c]->{'out'} = $res_out->{$key};
    $c++;
}
# Incoming
$c = 0;
foreach my $key (sort keys %$res_in) {
    $grupos[$c]->{'in'} = $res_in->{$key};
    $c++;
}
#print "\n";
#print Dumper @grupos;


## Mando la informacion ##
my $nsca_str;
my ($wth,$cth);
my ($status,$output);
my $nrdp_str;
foreach (0..$#grupos) {
    $status = $ERRORS{'OK'};
    $output = "OK";
    if ($opt_w && $opt_c) {
	$wth = &round($warn * $grupos[$_]->{size} / 100);
	$cth = &round($crit * $grupos[$_]->{size} / 100);
	if (($grupos[$_]->{in} >= $cth) || ($grupos[$_]->{out} >= $cth)) {
	    $status = $ERRORS{'CRITICAL'};
	    $output = "CRITICAL";
	} elsif (($grupos[$_]->{in} >= $wth) || ($grupos[$_]->{out} >= $wth)) {
	    $status = $ERRORS{'WARNING'};
	    $output = "WARNING";
	}
    }
    $output .= "|Max=" . $grupos[$_]->{size} . " Entrantes=" . $grupos[$_]->{in} . " Salientes=" . $grupos[$_]->{out} . " Total=" . ($grupos[$_]->{in} + $grupos[$_]->{out});
    #print "w: $wth c: $cth status $status\n";
    $tot_in += $grupos[$_]->{in};
    $tot_out += $grupos[$_]->{out};
    $tot_max += $grupos[$_]->{size};
    $nrdp_str = "$opt_H|" . $opt_p . $grupos[$_]->{name} . "|" . $status . "|$output\n";
    print "send_nrdp: $nrdp_str" if $debug;
    system("/usr/bin/send_nrdp.sh -u $opt_N -t $nrdp_token -H $opt_H -s \"$opt_p$grupos[$_]->{name}\" -S $status -o \"$output\"");
    print "\n" if $debug;
}

# Total
if (defined $res_grupos) {
    $status = $ERRORS{'OK'};
    $output = "OK|Max=$tot_max Entrantes=$tot_in Salientes=$tot_out" . " Total=" . ($tot_in+$tot_out);
} else {
    $status = $ERRORS{'UNKNOWN'};
    $output = "UNKNOWN|Max=$tot_max Entrantes=$tot_in Salientes=$tot_out" . " Total=" . ($tot_in+$tot_out);
}
$nrdp_str = "$opt_H|" . $opt_p . "Total" . "|" . $status . "|$output\n";
print "send_nrdp: $nrdp_str" if $debug;
system("/usr/bin/send_nrdp.sh -u $opt_N -t $nrdp_token -H $opt_H -s \"$opt_p" . "Total\" -S $status -o \"$output\"");
print "\n" if $debug;

# Llamadas locales
if (defined $res_locales) {
    $status = $ERRORS{'OK'};
    $output = "OK";
} else {
    $status = $ERRORS{'UNKNOWN'};
    $output = "UNKNOWN";
}
$output .= "|Internas=" . $res_locales->{$oid_lciCalls};
$nrdp_str = "$opt_H|LLAMADAS_Internas" . "|" . $status . "|$output\n";
print "send_nrdp: $nrdp_str" if $debug;
system("/usr/bin/send_nrdp.sh -u $opt_N -t $nrdp_token -H $opt_H -s \"LLAMADAS_Internas\" -S $status -o \"$output\"");
print "\n" if $debug;

# Uptime Jastserv
# up 182 days 22 hours 26 minutes | type=1 uptime_minutes=263426
my ($uptime_days,$uptime_hours,$uptime_mins) = (0,0,0);
my $uptime_str;
if (defined $res_uptime) {
    ($uptime_hours,$uptime_mins) = ($1,$2) if ($res_uptime->{$oid_serverUptime} =~ /^(\d+):(\d+):\d+$/);
    $uptime_days = int($uptime_hours / 24);
    $uptime_str = "up $uptime_days days " . ($uptime_hours % 24) . " hours $uptime_mins minutes";
    # print "uptime_mins: " . ($uptime_mins+$uptime_hours*60) ."\n";
    if (($uptime_mins+$uptime_hours*60) <= 5) {
	$status = $ERRORS{'CRITICAL'};
	$output = "CRITICAL: $uptime_str";
    } elsif (($uptime_mins+$uptime_hours*60) <= 15) {
	$status = $ERRORS{'WARNING'};
	$output = "WARNING: $uptime_str";
    } else {
	$status = $ERRORS{'OK'};
	$output = "OK: $uptime_str";
    }
    $output .= "|uptime_minutes=" . ($uptime_mins + $uptime_hours*60);
} else {
    $status = $ERRORS{'UNKNOWN'};
    $output = "UNKNOWN" ;
}
$nrdp_str = "$opt_H|IPCONTACT_Uptime|$status|$output\n";
print "send_nrdp: $nrdp_str" if $debug;
system("/usr/bin/send_nrdp.sh -u $opt_N -t $nrdp_token -H $opt_H -s \"IPCONTACT_Uptime\" -S $status -o \"$output\"");
print "\n" if $debug;

# Estado Licencia
if (defined $res_licence) {
    if ($res_licence->{$oid_licence_status} == 1) {
	$status = $ERRORS{'OK'};
	$output = "OK";
    } else {
	$status = $ERRORS{'CRITICAL'};
	$output = "CRITICAL";
    }
} else {
    $status = $ERRORS{'UNKNOWN'};
    $output = "UNKNOWN";
}
$output .= "|Valida=" . $res_licence->{$oid_licence_status};
$nrdp_str = "$opt_H|IPCONTACT_Licencia|$status|$output\n";
print "send_nrdp: $nrdp_str" if $debug;
system("/usr/bin/send_nrdp.sh -u $opt_N -t $nrdp_token -H $opt_H -s \"IPCONTACT_Licencia\" -S $status -o \"$output\"");
print "\n" if $debug;

sub print_usage () {
    print "\nUsage:\n";
    print "$PROGNAME\n";
    print "   -H (--hostname)      Nagios hostname (required, or read from ipsupervisor.conf)\n";
    print "   -N (--nsca)          NCSA host to send passive check (required, or read from ipsupervisor.conf)\n";
    print "   -S (--snmp)          SNMP hostname to query - (defaults to 'localhost')\n";
    print "   -P (--port)          NSCA port to send passive check (default 5667)\n";
    print "   -C (--community)     SNMP read community (defaults to 'ipcontact',\n";
    print "                        used with SNMP v1 and v2c\n";
    print "   -v (--snmp_version)  1 for SNMP v1 (default)\n";
    print "                        2 for SNMP v2c\n";
    print "   -p (--prefix)        Prefix to add to each line check (defaults 'LINEAS_')\n";
    print "   -d (--debug)         Enable NSCA debug (default disabled)\n";
    print "   -w (--warn)          Percentage of lines at which a warning message will be generated\n";
    print "                        ie: 80% (optional, default no warning generated)\n";
    print "   -c (--critical)       Percentage of lines at which a critical message will be generated\n";
    print "                        ie: 90% (optional, default no critical generated)\n";
    print "   -V (--version)       Plugin version\n";
    print "   -h (--help)          usage help\n";


}

